@@ -15,7 +15,7 @@ from os.path import isfile, join, relpath
 
 import jinja2
 import yaml
-from strictyaml import Any, Bool, Enum, Int, Map, MapPattern
+from strictyaml import Any, Bool, Enum, Float, Int, Map, MapPattern
 from strictyaml import Optional
 from strictyaml import Optional as Opt
 from strictyaml import Regex, Seq, Str, load
@@ -50,6 +50,8 @@ BASE_JOB_SCHEMA = {
     Optional("input_type"): Enum(config.known_input_types),
     Optional("dddb_tag"): Regex(RE_DDDB_TAG),
     Optional("conddb_tag"): Regex(RE_CONDDB_TAG),
+    Optional("checks"): Seq(Str()),  # TODO: replace this with a regex
+    Optional("extra_checks"): Seq(Str()),  # TODO: replace this with a regex
 }
 INPUT_SCHEMAS = {
     "bk_query": Map({"bk_query": Str(), Opt("n_test_lfns"): Int()}),
@@ -61,6 +63,55 @@ DEFAULT_JOB_VALUES = {
     "turbo": False,
 }
 
+CHECK_TYPE_SCHEMAS = {
+    "range": {
+        "expression": Str(),  # TODO: replace this with a regex
+        "limits": Map({"min": Float(), "max": Float()}),
+        "abs_tolerance": Float(),
+        Optional("blind_ranges"): Map({"min": Float(), "max": Float()})
+        | Seq(Map({"min": Float(), "max": Float()})),
+    },
+    "range_nd": {
+        "num_dimensions": Int(),
+        "expressions": Map(
+            {  # TODO: replace Str() with a regex
+                "x": Str(),
+                "y": Str(),
+                Optional("z"): Str(),
+                Optional("w"): Str(),
+            }
+        ),
+        "limits": Map(
+            {
+                "x": Map({"min": Float(), "max": Float()}),
+                "y": Map({"min": Float(), "max": Float()}),
+                Optional("z"): Map({"min": Float(), "max": Float()}),
+                Optional("w"): Map({"min": Float(), "max": Float()}),
+            }
+        ),
+        "abs_tolerance": Float(),
+        Optional("blind_ranges"): Seq(
+            Map(
+                {
+                    "x": Map({"min": Float(), "max": Float()}),
+                    "y": Map({"min": Float(), "max": Float()}),
+                    Optional("z"): Map({"min": Float(), "max": Float()}),
+                    Optional("w"): Map({"min": Float(), "max": Float()}),
+                }
+            )
+        ),
+    },
+    "num_entries": {
+        "count": Int(),
+    },
+    "num_entries_per_invpb": {
+        "count_per_invpb": Float(),
+    },
+}
+BASE_CHECK_SCHEMA = {
+    "type": Enum(list(CHECK_TYPE_SCHEMAS)),
+}
+
 
 def _ordered_dict_to_dict(a):
     if isinstance(a, (OrderedDict, dict)):
@@ -89,6 +140,20 @@ def parse_yaml(rendered_yaml):
         rendered_yaml, schema=MapPattern(Regex(RE_JOB_NAME), Any(), minimum_keys=1)
     )
 
+    data_checks = {}
+    if "checks" in data1:
+        # apply the appropriate schema to each different type of check
+        for _check_name, check_data in data1["checks"].items():
+            check_schema = {
+                **BASE_CHECK_SCHEMA,
+                **CHECK_TYPE_SCHEMAS[str(check_data["type"])],
+            }
+            check_data.revalidate(Map(check_schema))
+        # if checks pass validation, store elsewhere & delete from main data
+        # so that normal jobs aren't impacted
+        data_checks = data1.data["checks"]
+        del data1["checks"]
+
     if "defaults" in data1:
         defaults_schema = {}
         for key, value in BASE_JOB_SCHEMA.items():
@@ -146,9 +211,16 @@ def parse_yaml(rendered_yaml):
                 % (k, set(INPUT_SCHEMAS))
             )
 
-        data2.update(data.data)
+        # move contents of extra_checks to checks
+        data.data.setdefault("checks", [])
+        data_dict = data.data
+        if "extra_checks" in data_dict[k]:
+            data_dict[k]["checks"] += data_dict[k]["extra_checks"]
+            del data_dict[k]["extra_checks"]
 
-    return data2
+        data2.update(data_dict)
+
+    return data2, data_checks
 
 
 def _normalise_filetype(prod_name, job_name, filetype):
@@ -174,19 +246,24 @@ def _normalise_filetype(prod_name, job_name, filetype):
     return filetype
 
 
-def validate_yaml(data, repo_root, prod_name):
+def validate_yaml(jobs_data, checks_data, repo_root, prod_name):
     # Ensure all values that can be either a list or a string are lists of strings
-    for job_name, job_data in data.items():
+    for job_name, job_data in jobs_data.items():
         try:
-            _validate_job_data(repo_root, prod_name, job_name, job_data)
+            _validate_job_data(repo_root, prod_name, job_name, job_data, checks_data)
         except Exception as e:
             raise ValueError(f"Failed to validate {job_name!r} with error {e!r}")
+    # Validate checks
+    try:
+        _validate_checks_data(checks_data, jobs_data)
+    except Exception as e:
+        raise ValueError(f"Failed to validate checks with error {e!r}")
 
 
-def _validate_job_data(repo_root, prod_name, job_name, job_data):
+def _validate_job_data(repo_root, prod_name, job_name, job_data, checks_data):
     # Normalise list/str fields to always be lists
-    for prop in ["output", "options", "inform"]:
-        if not isinstance(job_data[prop], list):
+    for prop in ["output", "options", "inform", "checks", "extra_checks"]:
+        if not isinstance(job_data.get(prop, []), list):
             job_data[prop] = [job_data[prop]]
 
     # Validate the input data
@@ -217,3 +294,23 @@ def _validate_job_data(repo_root, prod_name, job_name, job_data):
             join("$ANALYSIS_PRODUCTIONS_BASE", prod_name, fn_normed)
         )
     job_data["options"] = normalised_options
+
+    # Validate the check names
+    # All checks listed for jobs should match a check defined in checks_data
+    if "checks" in job_data:
+        for ck in job_data["checks"]:
+            if ck not in list(checks_data.keys()):
+                raise ValueError(f"Check {ck} not found in list of defined checks")
+
+
+def _validate_checks_data(checks_data, jobs_data):
+    # All checks defined in checks_data should be used by at least 1 job
+    checks_used = {ck: False for ck in checks_data}
+    for _job_name, job_data in jobs_data.items():
+        if "checks" in job_data:
+            for ck in job_data["checks"]:
+                if ck in checks_used:
+                    checks_used[ck] = True
+    for ck, ck_used in checks_used.items():
+        if not ck_used:
+            raise ValueError(f"Check {ck} is defined but not used")
